/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 David Dribin All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Ant" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */



package org.dribin.dave.ant.j2me.taskdefs;

import java.io.*;

import org.apache.tools.ant.Task;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.util.*;
import org.apache.tools.ant.taskdefs.Java;
import org.apache.tools.ant.types.Commandline;

import org.dribin.dave.ant.j2me.util.*;

public class JarToPrc extends Java
{
    public void init()
    {
        super.init();

        mKvmHelper  = new KvmHelper(project);
    }

    public void setJarFile(File jarFile)
    {
        mJarFile    = jarFile;
    }

    public void setMainClass(String mainClass)
    {
        mMainClass  = mainClass;
    }

    public void setPrcFile(File prcFile)
    {
        mPrcFile    = prcFile;
    }

    public void setVerbose(boolean verbose)
    {
        mVerbose    = verbose;
    }

    public void setNetworking(boolean networking)
    {
        mNetworking = networking;
    }
    
    public void execute() throws BuildException
    {
        if (mJarFile == null)
        {
            throw new BuildException("jarfile attribute must be set",
                                     location);
        }
        
        if (! mJarFile.isFile())
        {
            throw new BuildException("jarfile \"" + mJarFile  +
                                     "\" does not exist or " +
                                     "is not a file",
                                     location);
        }
        
        if (mMainClass == null)
        {
            throw new BuildException("mainclass attribute must be set",
                                     location);
        }

        if (mPrcFile == null)
        {
            throw new BuildException("prcfile attribute must be set",
                                     location);
        }

        if (isPrcFileUpToDate())
        {
            return;
        }
        
        log("Building prc: " + mPrcFile.getAbsolutePath());

        setClasspath(mKvmHelper.getPalmToolsClassPath());
        setClassname("palm.database.MakePalmApp");
        createArg().setValue("-version");
        createArg().setValue("\"1.0\"");
        createArg().setValue("-bootclasspath");
        createArg().setPath(mKvmHelper.getCldcClassPath());

        if (mVerbose)
        {
            createArg().setValue("-v");
        }

        if (mNetworking)
        {
            createArg().setValue("-networking");
        }
        
        createArg().setValue("-outfile");
        createArg().setFile(mPrcFile);
        createArg().setValue("-JARtoPRC");
        createArg().setFile(mJarFile);
        createArg().setValue(mMainClass);
        setFork(true);

        super.execute();
    }

    private boolean isPrcFileUpToDate()
    {
        String[]    prcFilesToBuild;
        String[]    jarFiles;
        SourceFileScanner   scanner;
        FileNameMapper      jarToPrcMapper;

        scanner = new SourceFileScanner(this);

        jarFiles    = new String[1];
        jarFiles[0] = mJarFile.getName();

        jarToPrcMapper  = new GlobPatternMapper();
        jarToPrcMapper.setFrom("*.jar");
        jarToPrcMapper.setTo("*.prc");

        prcFilesToBuild = scanner.restrict(jarFiles,
                                           mJarFile.getParentFile(),
                                           mPrcFile.getParentFile(),
                                           jarToPrcMapper);

        log(prcFilesToBuild.length + " file(s) to build", Project.MSG_VERBOSE);
        if (prcFilesToBuild.length == 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    private KvmHelper               mKvmHelper;
    private File                    mJarFile;
    private String                  mMainClass;
    private File                    mPrcFile;
    private boolean                 mVerbose;
    private boolean                 mNetworking;
}
